@@ -14,6 +14,7 @@
 class DocumentManager;
 class FavoriteLocationModel;
 class HealthCertificateManager;
+class ImportController;
 class LiveDataManager;
 class PassManager;
 class PkPassManager;
@@ -26,8 +27,6 @@ class ExtractorDocumentNode;
 class File;
 }
 
-namespace KMime { class Message; }
-
 namespace KAndroidExtras {
 class Intent;
 }
@@ -61,12 +60,11 @@ public:
     void setTripGroupManager(TripGroupManager *tripGroupMgr);
     void setPassManager(PassManager *passMgr);
 
+    // page navigation intents
+    void handleIntent(const KAndroidExtras::Intent &intent);
+
     // data import
-    void importFromIntent(const KAndroidExtras::Intent &intent);
-    Q_INVOKABLE void importFromClipboard();
-    Q_INVOKABLE void importFromUrl(const QUrl &url);
-    Q_INVOKABLE bool importData(const QByteArray &data, const QString &fileName = {});
-    Q_INVOKABLE bool importText(const QString &text);
+    Q_INVOKABLE void commitImport(ImportController *importController);
 
     static ApplicationController* instance();
 
@@ -89,7 +87,7 @@ public:
     QString applicationVersion() const;
     QString extractorCapabilities() const;
     QVariant aboutData() const;
-    QString userAgent() const;
+    static QString userAgent();
 
     // health certificate manager
     bool hasHealthCertificateSupport() const;
@@ -104,21 +102,13 @@ Q_SIGNALS:
     /** Edit and add a new reservation. */
     void editNewHotelReservation(const QVariant &res);
     void editNewRestaurantReservation(const QVariant &res);
+    void editNewEventReservation(const QVariant &res);
 
 private:
-    static bool probablyUrl(const QString &text);
-    void importLocalFile(const QUrl &url);
-    bool importBundle(const QUrl &url);
-    bool importBundle(const QByteArray &data);
     bool importBundle(KItinerary::File *file);
-    bool importHealthCertificateRecursive(const KItinerary::ExtractorDocumentNode &node);
-    void importPass(const QString &passId);
-    void importMimeMessage(KMime::Message *msg);
-    void importNode(const KItinerary::ExtractorDocumentNode &node);
-    bool importGenericPkPass(const KItinerary::ExtractorDocumentNode &node);
+    void pkPassUpdated(const QString &passId);
 
     QString addDocumentFromFile(const QUrl &url);
-    QString addAttachableDocument(const QString &fileName, const QByteArray &data);
 
     bool exportTripToFile(const QString &tripGroupId, const QString &fileName);
     bool exportBatchToFile(const QString &batchId, const QString &fileName);
