@@ -9,7 +9,6 @@
 #include "logging.h"
 
 #include "applicationcontroller.h"
-#include "calendarimportmodel.h"
 #include "countrysubdivisionmodel.h"
 #include "clipboard.h"
 #include "developmentmodecontroller.h"
@@ -19,6 +18,8 @@
 #include "favoritelocationmodel.h"
 #include "genericpkpass.h"
 #include "healthcertificatemanager.h"
+#include "importcontroller.h"
+#include "intenthandler.h"
 #include "journeysectionmodel.h"
 #include "kdeconnect.h"
 #include "livedatamanager.h"
@@ -34,6 +35,7 @@
 #include "permissionmanager.h"
 #include "pkpassimageprovider.h"
 #include "publictransport.h"
+#include "reservationhelper.h"
 #include "reservationmanager.h"
 #include "settings.h"
 #include "statisticsmodel.h"
@@ -42,11 +44,15 @@
 #include "timelinedelegatecontroller.h"
 #include "timelinemodel.h"
 #include "timelinesectiondelegatecontroller.h"
+#include "traewellingcontroller.h"
 #include "transferdelegatecontroller.h"
 #include "transfermanager.h"
-#include "tripgroupinfoprovider.h"
+#include "tripgroupcontroller.h"
+#include "tripgroupfilterproxymodel.h"
 #include "tripgroupmanager.h"
+#include "tripgroupmodel.h"
 #include "tripgroupproxymodel.h"
+#include "tripgroupsplitmodel.h"
 #include "unitconversion.h"
 #include "util.h"
 #include "weatherforecastmodel.h"
@@ -59,9 +65,10 @@
 #include "weatherforecastmanager.h"
 
 #include "calendarlistmodel.h"
-#include "calendarpluginloader.h"
+#include <KCalendarCore/CalendarPluginLoader>
 
 #include <KItinerary/CountryDb>
+#include <KItinerary/DocumentUtil>
 #include <KItinerary/JsonLdDocument>
 #include <KItinerary/PriceUtil>
 #include <KItinerary/Ticket>
@@ -105,6 +112,8 @@
 #include <QStandardPaths>
 #include <QWindow>
 
+using namespace Qt::Literals::StringLiterals;
+
 #if !HAVE_MATRIX
 class MatrixBeaconStub : public QObject
 {
@@ -136,7 +145,7 @@ void registerKCalendarCoreTypes()
     qmlRegisterUncreatableMetaObject(KCalendarCore::staticMetaObject, "internal.org.kde.kcalendarcore", 1, 0, "KCalendarCore", {});
     qmlRegisterType<KCalendarCoreExtras::CalendarListModel>("internal.org.kde.kcalendarcore", 1, 0, "CalendarListModel");
     qmlRegisterSingletonType("internal.org.kde.kcalendarcore", 1, 0, "CalendarPluginLoader", [](QQmlEngine *, QJSEngine *jsEngine) -> QJSValue {
-        return jsEngine->toScriptValue(KCalendarCoreExtras::CalendarPluginLoader());
+        return jsEngine->toScriptValue(KCalendarCore::CalendarPluginLoader());
     });
 }
 
@@ -178,7 +187,6 @@ void registerApplicationTypes()
     qmlRegisterUncreatableMetaObject(MatrixRoomsModel::staticMetaObject, "org.kde.itinerary", 1, 0, "MatrixRoomsModel", {});
 #endif
 
-    qmlRegisterType<CalendarImportModel>("org.kde.itinerary", 1, 0, "CalendarImportModel");
     qmlRegisterType<CountrySubdivisionModel>("org.kde.itinerary", 1, 0, "CountrySubdivisionModel");
     qmlRegisterType<DocumentsModel>("org.kde.itinerary", 1, 0, "DocumentsModel");
     qmlRegisterType<JourneySectionModel>("org.kde.itinerary", 1, 0, "JourneySectionModel");
@@ -189,6 +197,9 @@ void registerApplicationTypes()
     qmlRegisterType<TimelineDelegateController>("org.kde.itinerary", 1, 0, "TimelineDelegateController");
     qmlRegisterType<TimelineSectionDelegateController>("org.kde.itinerary", 1, 0, "TimelineSectionDelegateController");
     qmlRegisterType<TransferDelegateController>("org.kde.itinerary", 1, 0, "TransferDelegateController");
+    qmlRegisterType<TripGroupController>("org.kde.itinerary", 1, 0, "TripGroupController");
+    qmlRegisterType<TripGroupFilterProxyModel>("org.kde.itinerary", 1, 0, "TripGroupFilterProxyModel");
+    qmlRegisterType<TripGroupSplitModel>("org.kde.itinerary", 1, 0, "TripGroupSplitModel");
     qmlRegisterType<WeatherForecastModel>("org.kde.itinerary", 1, 0, "WeatherForecastModel");
 #if HAVE_MATRIX
     qmlRegisterType<MatrixBeacon>("org.kde.itinerary", 1, 0, "MatrixBeacon");
@@ -196,14 +207,6 @@ void registerApplicationTypes()
     qmlRegisterType<MatrixBeaconStub>("org.kde.itinerary", 1, 0, "MatrixBeacon");
 #endif
     qmlRegisterType<OnlineTicketImporter>("org.kde.itinerary", 1, 0, "OnlineTicketImporter");
-
-    qmlRegisterSingletonType("org.kde.itinerary", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
-        return engine->toScriptValue(KAboutData::applicationData());
-    });
-
-    qmlRegisterSingletonType<Clipboard>("org.kde.itinerary", 1, 0, "Clipboard", [](QQmlEngine *, QJSEngine *) -> QObject * {
-        return new Clipboard;
-    });
 }
 
 // for registering QML singletons only
@@ -217,20 +220,17 @@ static TripGroupManager *s_tripGroupManager = nullptr;
 static LiveDataManager *s_liveDataMnager = nullptr;
 static WeatherForecastManager *s_weatherForecastManager = nullptr;
 static TimelineModel *s_timelineModel = nullptr;
-static TripGroupInfoProvider s_tripGroupInfoProvider;
 static TripGroupProxyModel *s_tripGroupProxyModel = nullptr;
 static MapDownloadManager *s_mapDownloadManager = nullptr;
 static PassManager *s_passManager = nullptr;
 static MatrixController *s_matrixController = nullptr;
+static ImportController *s_importController = nullptr;
+static TripGroupModel *s_tripGroupModel = nullptr;
+static TraewellingController *s_traewellingController = nullptr;
 
 #define REGISTER_SINGLETON_INSTANCE(Class, Instance) \
     qmlRegisterSingletonInstance<Class>("org.kde.itinerary", 1, 0, #Class, Instance);
 
-#define REGISTER_SINGLETON_GADGET_INSTANCE(Class, Instance) \
-    qmlRegisterSingletonType("org.kde.itinerary", 1, 0, #Class, [](QQmlEngine *engine, QJSEngine*) -> QJSValue { \
-        return engine->toScriptValue(Instance); \
-    });
-
 #define REGISTER_SINGLETON_GADGET_FACTORY(Class) \
     qmlRegisterSingletonType("org.kde.itinerary", 1, 0, #Class, [](QQmlEngine*, QJSEngine *engine) -> QJSValue { \
         return engine->toScriptValue(Class()); \
@@ -253,8 +253,9 @@ void registerApplicationSingletons()
     REGISTER_SINGLETON_INSTANCE(MapDownloadManager, s_mapDownloadManager)
     REGISTER_SINGLETON_INSTANCE(PassManager, s_passManager)
     REGISTER_SINGLETON_INSTANCE(MatrixController, s_matrixController);
-
-    REGISTER_SINGLETON_GADGET_INSTANCE(TripGroupInfoProvider, s_tripGroupInfoProvider)
+    REGISTER_SINGLETON_INSTANCE(ImportController, s_importController);
+    REGISTER_SINGLETON_INSTANCE(TripGroupModel, s_tripGroupModel);
+    REGISTER_SINGLETON_INSTANCE(TraewellingController, s_traewellingController);
 
     REGISTER_SINGLETON_GADGET_FACTORY(DevelopmentModeController)
     REGISTER_SINGLETON_GADGET_FACTORY(Factory)
@@ -263,12 +264,20 @@ void registerApplicationSingletons()
     REGISTER_SINGLETON_GADGET_FACTORY(NotificationConfigController)
     REGISTER_SINGLETON_GADGET_FACTORY(PermissionManager)
     REGISTER_SINGLETON_GADGET_FACTORY(PublicTransport)
+    REGISTER_SINGLETON_GADGET_FACTORY(ReservationHelper)
     REGISTER_SINGLETON_GADGET_FACTORY(UnitConversion)
     REGISTER_SINGLETON_GADGET_FACTORY(Util)
+
+    qmlRegisterSingletonType("org.kde.itinerary", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
+        return engine->toScriptValue(KAboutData::applicationData());
+    });
+
+    qmlRegisterSingletonType<Clipboard>("org.kde.itinerary", 1, 0, "Clipboard", [](QQmlEngine *, QJSEngine *) -> QObject * {
+        return new Clipboard;
+    });
 }
 
 #undef REGISTER_SINGLETON_INSTANCE
-#undef REGISTER_SINGLETON_GADGET_INSTANCE
 #undef REGISTER_SINGLETON_GADGET_FACTORY
 
 static QNetworkAccessManager *namFactory()
@@ -283,17 +292,18 @@ static QNetworkAccessManager *namFactory()
     return s_nam;
 }
 
-void handleCommandLineArguments(ApplicationController *appController, const QStringList &args, bool isTemporary, const QString &page)
+void handleCommandLineArguments(ApplicationController *appController, ImportController *importController, const QStringList &args, bool isTemporary, const QString &page)
 {
     for (const auto &file : args) {
+        importController->setAutoCommitEnablted(!s_settings->developmentMode());
         const auto localUrl = QUrl::fromLocalFile(file);
         if (QFile::exists(localUrl.toLocalFile())) {
-            appController->importFromUrl(localUrl);
+            importController->importFromUrl(localUrl);
             if (isTemporary) {
                 QFile::remove(localUrl.toLocalFile());
             }
         } else {
-            appController->importFromUrl(QUrl::fromUserInput(file));
+            importController->importFromUrl(QUrl::fromUserInput(file));
         }
     }
 
@@ -339,22 +349,24 @@ int main(int argc, char **argv)
     QCommandLineParser parser;
     QCommandLineOption isTemporaryOpt(QStringLiteral("tempfile"), QStringLiteral("Input file is a temporary file and will be deleted after importing."));
     parser.addOption(isTemporaryOpt);
-    QCommandLineOption pageOpt(QStringLiteral("page"), i18n("Open Itinerary on the given page"), QStringLiteral("page"));
+    QCommandLineOption pageOpt(QStringLiteral("page"), i18nc("@info:shell", "Open Itinerary on the given page"), QStringLiteral("page"));
     parser.addOption(pageOpt);
     QCommandLineOption selfTestOpt(QStringLiteral("self-test"), QStringLiteral("internal, for automated testing"));
     parser.addOption(selfTestOpt);
     aboutData.setupCommandLine(&parser);
-    parser.addPositionalArgument(QStringLiteral("file"), i18n("Files or URLs to import."));
+    parser.addPositionalArgument(QStringLiteral("file"), i18nc("@info:shell", "Files or URLs to import."));
     parser.process(app);
     aboutData.processCommandLine(&parser);
 #if HAVE_KCRASH
     KCrash::initialize();
 #endif
 
-#ifndef Q_OS_ANDROID
+#if !defined(Q_OS_ANDROID) && !defined(Q_OS_HAIKU)
     KDBusService service(KDBusService::Unique);
 #endif
 
+    IntentHandler intentHandler;
+
     Settings settings;
     s_settings = &settings;
 
@@ -391,7 +403,6 @@ int main(int argc, char **argv)
 
     TransferManager transferManager;
     transferManager.setReservationManager(&resMgr);
-    transferManager.setTripGroupManager(&tripGroupMgr);
     transferManager.setFavoriteLocationModel(&favLocModel);
     transferManager.setLiveDataManager(&liveDataMgr);
     transferManager.setAutoAddTransfers(settings.autoAddTransfers());
@@ -402,6 +413,10 @@ int main(int argc, char **argv)
 
     tripGroupMgr.setTransferManager(&transferManager);
 
+    TripGroupModel tripGroupModel;
+    tripGroupModel.setTripGroupManager(&tripGroupMgr);
+    s_tripGroupModel = &tripGroupModel;
+
     TimelineModel timelineModel;
     timelineModel.setHomeCountryIsoCode(settings.homeCountryIsoCode());
     timelineModel.setReservationManager(&resMgr);
@@ -415,9 +430,6 @@ int main(int argc, char **argv)
     tripGroupProxy.setSourceModel(&timelineModel);
     s_tripGroupProxyModel = &tripGroupProxy;
 
-    s_tripGroupInfoProvider.setReservationManager(&resMgr);
-    s_tripGroupInfoProvider.setWeatherForecastManager(&weatherForecastMgr);
-
     MapDownloadManager mapDownloadMgr;
     mapDownloadMgr.setReservationManager(&resMgr);
     mapDownloadMgr.setAutomaticDownloadEnabled(s_settings->preloadMapData());
@@ -431,6 +443,15 @@ int main(int argc, char **argv)
     MatrixController matrixController;
     s_matrixController = &matrixController;
 
+    ImportController importController;
+    importController.setNetworkAccessManagerFactory(namFactory);
+    importController.setReservationManager(&resMgr);
+    QObject::connect(&intentHandler, &IntentHandler::handleIntent, &importController, &ImportController::importFromIntent);
+    s_importController = &importController;
+
+    TraewellingController traewellingController(namFactory);
+    s_traewellingController = &traewellingController;
+
     ApplicationController appController;
     appController.setNetworkAccessManagerFactory(namFactory);
     appController.setReservationManager(&resMgr);
@@ -441,13 +462,13 @@ int main(int argc, char **argv)
     appController.setLiveDataManager(&liveDataMgr);
     appController.setTripGroupManager(&tripGroupMgr);
     appController.setPassManager(&passMgr);
-#ifndef Q_OS_ANDROID
+#if !defined(Q_OS_ANDROID) && !defined(Q_OS_HAIKU)
     QObject::connect(&service, &KDBusService::activateRequested, [&](const QStringList &args, const QString &workingDir) {
         qCDebug(Log) << "remote activation" << args << workingDir;
         if (!args.isEmpty()) {
             QDir::setCurrent(workingDir);
             parser.parse(args);
-            handleCommandLineArguments(&appController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
+            handleCommandLineArguments(&appController, &importController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
         }
         if (!QGuiApplication::allWindows().isEmpty()) {
             QWindow *window = QGuiApplication::allWindows().at(0);
@@ -456,6 +477,8 @@ int main(int argc, char **argv)
         }
     });
 #endif
+    QObject::connect(&intentHandler, &IntentHandler::handleIntent, &appController, &ApplicationController::handleIntent);
+    QObject::connect(&importController, &ImportController::infoMessage, &appController, &ApplicationController::infoMessage);
 
     OnlineTicketImporter::setNetworkAccessManagerFactory(namFactory);
 
@@ -467,7 +490,21 @@ int main(int argc, char **argv)
     registerApplicationSingletons();
 
     QQmlApplicationEngine engine;
-    engine.addImageProvider(QStringLiteral("org.kde.pkpass"), new PkPassImageProvider(&pkPassMgr));
+
+    auto pkPassImageProvider = new PkPassImageProvider;
+    pkPassImageProvider->registerPassProvider([&pkPassMgr](const QString &passTypeId, const QString &serialNum) -> KPkPass::Pass* {
+        return pkPassMgr.pass(passTypeId + '/'_L1 + QString::fromUtf8(serialNum.toUtf8().toBase64(QByteArray::Base64UrlEncoding)));
+    });
+    pkPassImageProvider->registerPassProvider([&importController](const QString &passTypeId, const QString &serialNum) -> KPkPass::Pass* {
+        if (const auto it = importController.pkPasses().find(KItinerary::DocumentUtil::idForPkPass(passTypeId, serialNum)); it != importController.pkPasses().end()) {
+            if (!(*it).second.pass) {
+                (*it).second.pass.reset(KPkPass::Pass::fromData((*it).second.data));
+            }
+            return (*it).second.pass.get();
+        }
+        return nullptr;
+    });    engine.addImageProvider(QStringLiteral("org.kde.pkpass"), pkPassImageProvider);
+
     auto l10nContext = new KLocalizedContext(&engine);
     l10nContext->setTranslationDomain(QStringLiteral(TRANSLATION_DOMAIN));
     engine.rootContext()->setContextObject(l10nContext);
@@ -478,11 +515,11 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    handleCommandLineArguments(&appController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
+    handleCommandLineArguments(&appController, &importController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
 
 #ifdef Q_OS_ANDROID
     using namespace KAndroidExtras;
-    appController.importFromIntent(Activity::getIntent());
+    intentHandler.handleIntent(Activity::getIntent());
 #endif
 
     if (parser.isSet(selfTestOpt)) {
