@@ -15,6 +15,8 @@
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Stopover>
 
+#include <QTimeZone>
+
 bool PublicTransportMatcher::isSameMode(const QVariant &res, KPublicTransport::Line::Mode mode)
 {
     using namespace KPublicTransport;
@@ -91,12 +93,12 @@ static bool isSameDateTime(const QDateTime &lhs, const QDateTime &rhs)
     }
     if (lhs.timeSpec() == Qt::LocalTime && rhs.timeSpec() != Qt::LocalTime) {
         QDateTime dt(rhs);
-        dt.setTimeSpec(Qt::LocalTime);
+        dt.setTimeZone(QTimeZone::LocalTime);
         return lhs == dt;
     }
     if (lhs.timeSpec() != Qt::LocalTime && rhs.timeSpec() == Qt::LocalTime) {
         QDateTime dt(lhs);
-        dt.setTimeSpec(Qt::LocalTime);
+        dt.setTimeZone(QTimeZone::LocalTime);
         return dt == rhs;
     }
     return false;
