@@ -12,9 +12,12 @@
 #include <QDebug>
 #include <QJsonObject>
 
+using namespace Qt::Literals::StringLiterals;
+
 class TransferPrivate : public QSharedData
 {
 public:
+    // ### any update here needs to also be reflected in the operator== implementation below
     Transfer::Alignment m_alignment = Transfer::Before;
     Transfer::State m_state = Transfer::UndefinedState;
     KPublicTransport::Location m_from;
@@ -95,8 +98,9 @@ void Transfer::setTo(const KPublicTransport::Location &to)
 
 QString Transfer::fromName() const
 {
-    if (d->m_fromName.isEmpty())
+    if (d->m_fromName.isEmpty()) {
         return d->m_from.name();
+    }
     return d->m_fromName;
 }
 
@@ -108,8 +112,9 @@ void Transfer::setFromName(const QString &fromName)
 
 QString Transfer::toName() const
 {
-    if (d->m_toName.isEmpty())
+    if (d->m_toName.isEmpty()) {
         return d->m_to.name();
+    }
     return d->m_toName;
 }
 
@@ -124,6 +129,11 @@ bool Transfer::hasLocations() const
     return !d->m_from.isEmpty() && !d->m_to.isEmpty();
 }
 
+bool Transfer::hasCoordinates() const
+{
+    return d->m_from.hasCoordinate() && d->m_to.hasCoordinate();
+}
+
 KPublicTransport::Journey Transfer::journey() const
 {
     return d->m_journey;
@@ -135,9 +145,9 @@ void Transfer::setJourney(const KPublicTransport::Journey &journey)
     d->m_journey = journey;
     if (journey.scheduledArrivalTime().isValid()) {
         if (d->m_alignment == Transfer::Before) {
-            d->m_anchorDelta = std::max(0ll, journey.scheduledArrivalTime().secsTo(d->m_anchorTime));
+            d->m_anchorDelta = (int)std::max(0ll, journey.scheduledArrivalTime().secsTo(d->m_anchorTime));
         } else {
-            d->m_anchorDelta = std::max(0ll, d->m_anchorTime.secsTo(journey.scheduledDepartureTime()));
+            d->m_anchorDelta = (int)std::max(0ll, d->m_anchorTime.secsTo(journey.scheduledDepartureTime()));
         }
     }
 }
@@ -207,24 +217,45 @@ QString Transfer::identifier() const
 
 QString Transfer::identifier(const QString &resId, Transfer::Alignment alignment)
 {
-    return resId + (alignment == Transfer::Before ? QLatin1StringView("-BEFORE") : QLatin1StringView("-AFTER"));
+    return resId + (alignment == Transfer::Before ? "-BEFORE"_L1 : "-AFTER"_L1);
+}
+
+bool Transfer::operator==(const Transfer &other) const
+{
+    if (d->m_alignment == other.d->m_alignment
+     && d->m_state == other.d->m_state
+     && KPublicTransport::Location::isSame(d->m_from, other.d->m_from)
+     && KPublicTransport::Location::isSame(d->m_to, other.d->m_to)
+     && KPublicTransport::Journey::isSame(d->m_journey, other.d->m_journey)
+     && d->m_fromName == other.d->m_fromName
+     && d->m_toName == other.d->m_toName
+     && d->m_anchorTime == other.d->m_anchorTime
+     && d->m_anchorDelta == other.d->m_anchorDelta
+     && d->m_floatingLocationType == other.d->m_floatingLocationType)
+    {
+        if (d->m_state == Transfer::Selected) {
+            return false; // we lack a way to check if m_journey is exactly identical!
+        }
+        return true;
+    }
+    return false;
 }
 
 QJsonObject Transfer::toJson(const Transfer &transfer)
 {
     auto obj = Json::toJson(transfer);
-    obj.insert(QStringLiteral("from"), KPublicTransport::Location::toJson(transfer.from()));
-    obj.insert(QStringLiteral("to"), KPublicTransport::Location::toJson(transfer.to()));
-    obj.insert(QStringLiteral("journey"), KPublicTransport::Journey::toJson(transfer.journey()));
+    obj.insert("from"_L1, KPublicTransport::Location::toJson(transfer.from()));
+    obj.insert("to"_L1, KPublicTransport::Location::toJson(transfer.to()));
+    obj.insert("journey"_L1, KPublicTransport::Journey::toJson(transfer.journey()));
     return obj;
 }
 
 Transfer Transfer::fromJson(const QJsonObject &obj)
 {
     auto transfer = Json::fromJson<Transfer>(obj);
-    transfer.setFrom(KPublicTransport::Location::fromJson(obj.value(QLatin1StringView("from")).toObject()));
-    transfer.setTo(KPublicTransport::Location::fromJson(obj.value(QLatin1StringView("to")).toObject()));
-    transfer.setJourney(KPublicTransport::Journey::fromJson(obj.value(QLatin1StringView("journey")).toObject()));
+    transfer.setFrom(KPublicTransport::Location::fromJson(obj.value("from"_L1).toObject()));
+    transfer.setTo(KPublicTransport::Location::fromJson(obj.value("to"_L1).toObject()));
+    transfer.setJourney(KPublicTransport::Journey::fromJson(obj.value("journey"_L1).toObject()));
 
     if (transfer.state() == Searching) { // searching state cannot survive persisting
         transfer.setState(Pending);
