@@ -11,6 +11,7 @@
 #include "reservationmanager.h"
 #include "transfermanager.h"
 
+#include <KItinerary/Event>
 #include <KItinerary/LocationUtil>
 #include <KItinerary/Organization>
 #include <KItinerary/Reservation>
@@ -26,13 +27,12 @@
 
 #include <set>
 
+using namespace Qt::Literals::StringLiterals;
 using namespace KItinerary;
 
-enum {
-    MaximumTripDuration = 20, // in days
-    MaximumTripElements = 20,
-    MinimumTripElements = 2,
-};
+constexpr inline const auto MaximumTripDuration = 20; // in days
+constexpr inline const auto MaximumTripElements = 20;
+constexpr inline const auto MinimumTripElements = 2;
 
 TripGroupManager::TripGroupManager(QObject* parent) :
     QObject(parent)
@@ -42,6 +42,11 @@ TripGroupManager::TripGroupManager(QObject* parent) :
 
 TripGroupManager::~TripGroupManager() = default;
 
+ReservationManager* TripGroupManager::reservationManager() const
+{
+    return m_resMgr;
+}
+
 void TripGroupManager::setReservationManager(ReservationManager *resMgr)
 {
     m_resMgr = resMgr;
@@ -58,21 +63,47 @@ void TripGroupManager::setReservationManager(ReservationManager *resMgr)
         return SortUtil::isBefore(m_resMgr->reservation(lhs), m_resMgr->reservation(rhs));
     });
 
-    checkConsistency();
-    scanAll();
+    if (m_resMgr && m_transferMgr) {
+        checkConsistency();
+        scanAll();
+    }
 }
 
 void TripGroupManager::setTransferManager(TransferManager *transferMgr)
 {
     m_transferMgr = transferMgr;
+
+    const auto transferChangedWrapper = [this](const Transfer &t) { transferChanged(t.reservationId(), t.alignment()); };
+    connect(m_transferMgr, &TransferManager::transferAdded, this, transferChangedWrapper);
+    connect(m_transferMgr, &TransferManager::transferChanged, this, transferChangedWrapper);
+    connect(m_transferMgr, &TransferManager::transferRemoved, this, &TripGroupManager::transferChanged);
+
+    if (m_resMgr && m_transferMgr) {
+        checkConsistency();
+        scanAll();
+    }
 }
 
-QList<QString> TripGroupManager::tripGroups() const {
-  QList<QString> groups;
-  groups.reserve(m_tripGroups.size());
-  std::copy(m_tripGroups.keyBegin(), m_tripGroups.keyEnd(),
-            std::back_inserter(groups));
-  return groups;
+void TripGroupManager::suspend()
+{
+    m_suspended = true;
+    m_shouldScan = false;
+}
+
+void TripGroupManager::resume()
+{
+    m_suspended = false;
+    if (m_shouldScan) {
+        scanAll();
+    }
+}
+
+std::vector<QString> TripGroupManager::tripGroups() const
+{
+    std::vector<QString> groups;
+    groups.reserve(m_tripGroups.size());
+    std::copy(m_tripGroups.keyBegin(), m_tripGroups.keyEnd(), std::back_inserter(groups));
+    return groups;
 }
 
 TripGroup TripGroupManager::tripGroup(const QString &id) const
@@ -92,7 +123,12 @@ TripGroup TripGroupManager::tripGroupForReservation(const QString &resId) const
 
 QString TripGroupManager::basePath()
 {
-    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QStringLiteral("/tripgroups/");
+    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/tripgroups/"_L1;
+}
+
+QString TripGroupManager::fileForGroup(QStringView tgId)
+{
+    return basePath() + tgId + ".json"_L1;
 }
 
 void TripGroupManager::load()
@@ -102,7 +138,7 @@ void TripGroupManager::load()
 
     for (QDirIterator it(base, QDir::NoDotAndDotDot | QDir::Files); it.hasNext();) {
         it.next();
-        TripGroup g(this);
+        TripGroup g;
         if (g.load(it.filePath())) {
             const auto tgId = it.fileInfo().baseName();
             m_tripGroups.insert(tgId, g);
@@ -110,7 +146,7 @@ void TripGroupManager::load()
                 const auto groupIt = m_reservationToGroupMap.constFind(resId);
                 if (groupIt != m_reservationToGroupMap.constEnd()) {
                     qCWarning(Log) << "Overlapping trip groups found - removing" << g.name();
-                    const auto groupId = groupIt.value(); // copy before we modify what groupIt points to
+                    const auto groupId = groupIt.value(); // copy before we modify what groupIt points to - NOLINT performance-unnecessary-copy-initialization
                     removeTripGroup(groupId);
                     removeTripGroup(tgId);
                     break;
@@ -136,7 +172,7 @@ void TripGroupManager::removeTripGroup(const QString &groupId)
         }
     }
     m_tripGroups.erase(groupIt);
-    if (!QFile::remove(basePath() + groupId + QLatin1StringView(".json"))) {
+    if (!QFile::remove(fileForGroup(groupId))) {
         qCWarning(Log) << "Failed to delete trip group file!" << groupId;
     }
     Q_EMIT tripGroupRemoved(groupId);
@@ -149,6 +185,19 @@ void TripGroupManager::clear()
     d.removeRecursively();
 }
 
+void TripGroupManager::updateTripGroup(const QString &groupId, const TripGroup &group)
+{
+    auto groupIt = m_tripGroups.find(groupId);
+    if (groupIt == m_tripGroups.end()) {
+        return;
+    }
+
+    groupIt.value() = group;
+    recomputeTripGroupTimes(groupIt.value());
+    groupIt.value().store(fileForGroup(groupId));
+    Q_EMIT tripGroupChanged(groupId);
+}
+
 void TripGroupManager::removeReservationsInGroup(const QString &groupId)
 {
     const auto groupIt = m_tripGroups.constFind(groupId);
@@ -156,10 +205,18 @@ void TripGroupManager::removeReservationsInGroup(const QString &groupId)
         return;
     }
 
+    // as we remove entries one by one we'd get and propagate update notifications for all of them
+    // so block those and emit one notifications manually once we are done
+    TripGroupingBlocker groupingBlocker(this);
+    QSignalBlocker signalBlocker(this);
+
     const auto elements = groupIt.value().elements();
     for (const auto &element : elements) {
         m_resMgr->removeBatch(element);
     }
+
+    signalBlocker.unblock();
+    Q_EMIT tripGroupRemoved(groupId);
 }
 
 void TripGroupManager::batchAdded(const QString &resId)
@@ -174,16 +231,45 @@ void TripGroupManager::batchAdded(const QString &resId)
 
 void TripGroupManager::batchContentChanged(const QString &resId)
 {
-    // ### we can probably make this more efficient
-    batchRemoved(resId);
-    batchAdded(resId);
+    const auto tgId = tripGroupIdForReservation(resId);
+    auto tg = tripGroup(tgId);
+    if (!tg.isAutomaticallyGrouped()) {
+        // don't touch the grouping, just deal with potential time changes
+        auto elements = tg.elements();
+        std::sort(elements.begin(), elements.end(), [this](const auto &lhs, const auto &rhs) {
+            return SortUtil::isBefore(m_resMgr->reservation(lhs), m_resMgr->reservation(rhs));
+        });
+        tg.setElements(elements);
+        recomputeTripGroupTimes(tg);
+        tg.store(fileForGroup(tgId));
+        Q_EMIT tripGroupChanged(tgId);
+    } else {
+        // ### we can probably make this more efficient
+        batchRemoved(resId);
+        batchAdded(resId);
+    }
 }
 
 void TripGroupManager::batchRenamed(const QString &oldBatchId, const QString &newBatchId)
 {
-    // ### this can be done more efficiently
-    batchRemoved(oldBatchId);
-    batchAdded(newBatchId);
+    const auto mapIt = m_reservationToGroupMap.constFind(oldBatchId);
+    if (mapIt == m_reservationToGroupMap.end()) {
+        return;
+    }
+
+    const auto tgId = mapIt.value(); // NOLINT performance-unnecessary-copy-initialization
+
+    auto tg = tripGroup(tgId);
+    auto elems = tg.elements();
+    const auto elemIt = std::find(elems.begin(), elems.end(), oldBatchId);
+    (*elemIt) = newBatchId;
+    tg.setElements(elems);
+
+    m_reservationToGroupMap.erase(mapIt);
+    m_reservationToGroupMap[newBatchId] = tgId;
+
+    tg.store(fileForGroup(tgId));
+    Q_EMIT tripGroupChanged(tgId);
 }
 
 void TripGroupManager::batchRemoved(const QString &resId)
@@ -193,17 +279,18 @@ void TripGroupManager::batchRemoved(const QString &resId)
     if (mapIt != m_reservationToGroupMap.constEnd()) {
         const auto groupIt = m_tripGroups.find(mapIt.value());
         Q_ASSERT(groupIt != m_tripGroups.end());
-        const auto groupId = groupIt.key(); // copy as the iterator might become invalid below
+        const auto groupId = groupIt.key(); // copy as the iterator might become invalid below - NOLINT performance-unnecessary-copy-initialization
 
         auto elems = groupIt.value().elements();
         elems.removeAll(resId);
-        if (elems.size() < MinimumTripElements) { // group deleted
+        if (elems.isEmpty() || (groupIt.value().isAutomaticallyGrouped() && elems.size() < MinimumTripElements)) { // group deleted
             qDebug() << "removing trip group due to getting too small";
             removeTripGroup(groupId);
         } else { // group changed
             qDebug() << "removing element from trip group" << resId << elems;
             groupIt.value().setElements(elems);
-            groupIt.value().store(basePath() + mapIt.value() + QLatin1StringView(".json"));
+            recomputeTripGroupTimes(groupIt.value());
+            groupIt.value().store(fileForGroup(mapIt.value()));
             m_reservationToGroupMap.erase(mapIt);
             Q_EMIT tripGroupChanged(groupId);
         }
@@ -216,8 +303,33 @@ void TripGroupManager::batchRemoved(const QString &resId)
     }
 }
 
+void TripGroupManager::transferChanged(const QString &resId, Transfer::Alignment alignment)
+{
+    const auto tgId = m_reservationToGroupMap.value(resId);
+    if (tgId.isEmpty()) {
+        return;
+    }
+
+    const auto tgIt = m_tripGroups.find(tgId);
+    assert(tgIt != m_tripGroups.end() && !tgIt.value().elements().empty());
+
+    // if tranfers leading or trailing the trip group changed, the start/end time of the group change
+    if ((alignment == Transfer::Before && tgIt.value().elements().constFirst() == resId)
+     || (alignment == Transfer::After && tgIt.value().elements().constLast() == resId)) {
+        if (recomputeTripGroupTimes(tgIt.value())) {
+            tgIt.value().store(fileForGroup(tgIt.key()));
+            Q_EMIT tripGroupChanged(tgId);
+        }
+    }
+}
+
 void TripGroupManager::scanAll()
 {
+    if (m_suspended) {
+        m_shouldScan = true;
+        return;
+    }
+
     qCDebug(Log);
     QString prevGroup;
     for (auto it = m_reservations.begin(); it != m_reservations.end(); ++it) {
@@ -267,17 +379,28 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         m_resNumSearch.push_back({beginRes.userType(), JsonLd::convert<Reservation>(beginRes).reservationNumber()});
     }
 
+    const auto prevTgId = tripGroupIdForReservation(*beginIt);
+    const auto prevTg = tripGroup(prevTgId);
+    const auto explicitEnd = !prevTg.isAutomaticallyGrouped() && !prevTg.elements().empty() ? prevTg.elements().constLast() : QString();
+
     qDebug() << "starting scan at" << LocationUtil::name(beginDeparture);
     auto res = beginRes;
     auto resNumIt = m_reservations.cend(); // result of the search using reservation ids
     auto connectedIt = m_reservations.cend(); // result of the search using trip connectivity
+    auto explicitIt = m_reservations.cend(); // result of the search using an existing explicitly managed group
 
     bool resNumSearchDone = false;
     bool connectedSearchDone = false;
+    bool explicitSearchDone = explicitEnd.isEmpty();
     bool reachedStartAgain = false;
 
     // scan by location change
     for (auto it = beginIt + 1; it != m_reservations.end(); ++it) {
+        // if we reached the next manually grouped element here, stop immediately
+        if (const auto tgId = tripGroupIdForReservation(*it); !tgId.isEmpty() && tgId != prevTgId && !tripGroup(tgId).isAutomaticallyGrouped()) {
+            break;
+        }
+
         const auto prevRes = res;
         const auto curRes = m_resMgr->reservation(*it);
 
@@ -288,20 +411,20 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         res = curRes;
 
         // all search strategies think they are done
-        if (resNumSearchDone && connectedSearchDone) {
+        if (resNumSearchDone && connectedSearchDone && explicitSearchDone) {
             break;
         }
 
         // search depth reached
         // ### we probably don't want to count multi-traveler elements for this!
-        if (std::distance(beginIt, it) > MaximumTripElements) {
+        if (explicitSearchDone && std::distance(beginIt, it) > MaximumTripElements) {
             qDebug() << "  aborting search, maximum search depth reached";
             break;
         }
 
         // maximum trip duration exceeded?
         const auto endDt = SortUtil::endDateTime(res);
-        if (beginDt.daysTo(endDt) > MaximumTripDuration) {
+        if (explicitSearchDone && beginDt.daysTo(endDt) > MaximumTripDuration) {
             qDebug() << "  aborting search, maximum trip duration reached";
             break;
         }
@@ -360,6 +483,11 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
                 resNumSearchDone = true;
             }
         }
+
+        if (!explicitSearchDone && (*it) == explicitEnd) {
+            explicitIt = it;
+            explicitSearchDone = true;
+        }
     }
 
     // determine which search strategy found the larger result
@@ -372,6 +500,9 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
     } else {
         it = connectedIt == m_reservations.end() ? resNumIt : connectedIt;
     }
+    if (explicitIt != m_reservations.cend()) {
+        it = it == m_reservations.end() ? explicitIt : std::max(it, explicitIt);
+    }
 
     if (it == m_reservations.end()) {
         qDebug() << "nothing found";
@@ -379,20 +510,22 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
     }
 
     // remove leading loop appendices (trailing ones will be cut by the loop check above already)
-    const auto endRes = m_resMgr->reservation(*it);
-    const auto endArrival = LocationUtil::arrivalLocation(endRes);
-    for (auto it2 = beginIt; it2 != it; ++it2) {
-        const auto res = m_resMgr->reservation(*it2);
-        if (!LocationUtil::isLocationChange(res)) {
-            continue;
-        }
-        const auto curDeparture = LocationUtil::departureLocation(res);
-        if (LocationUtil::isSameLocation(endArrival, curDeparture, LocationUtil::CityLevel)) {
-            if (beginIt != it2) {
-                qDebug() << "  removing leading appendix, starting at" << LocationUtil::name(curDeparture);
+    if (prevTg.isAutomaticallyGrouped()) {
+        const auto endRes = m_resMgr->reservation(*it);
+        const auto endArrival = LocationUtil::arrivalLocation(endRes);
+        for (auto it2 = beginIt; it2 != it; ++it2) {
+            const auto res = m_resMgr->reservation(*it2);
+            if (!LocationUtil::isLocationChange(res)) {
+                continue;
+            }
+            const auto curDeparture = LocationUtil::departureLocation(res);
+            if (LocationUtil::isSameLocation(endArrival, curDeparture, LocationUtil::CityLevel)) {
+                if (beginIt != it2) {
+                    qDebug() << "  removing leading appendix, starting at" << LocationUtil::name(curDeparture);
+                }
+                beginIt = it2;
+                break;
             }
-            beginIt = it2;
-            break;
         }
     }
 
@@ -417,7 +550,7 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
     std::set<QString> pendingGroupRemovals;
     if (groupIt == m_tripGroups.end()) {
         const auto tgId = QUuid::createUuid().toString(QUuid::WithoutBraces);
-        TripGroup g(this);
+        TripGroup g;
         g.setElements(elems);
         for (auto it2 = beginIt; it2 != it; ++it2) {
             // remove overlapping/nested groups, delay this until the end though, as that will invalidate our iterators
@@ -427,10 +560,11 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
             }
             m_reservationToGroupMap.insert(*it2, tgId);
         }
-        g.setName(guessName(g));
+        g.setName(guessName(g.elements()));
+        recomputeTripGroupTimes(g);
         qDebug() << "creating trip group" << g.name();
         m_tripGroups.insert(tgId, g);
-        g.store(basePath() + tgId + QLatin1StringView(".json"));
+        g.store(fileForGroup(tgId));
         Q_EMIT tripGroupAdded(tgId);
     } else {
         auto &g = groupIt.value();
@@ -441,9 +575,12 @@ void TripGroupManager::scanOne(std::vector<QString>::const_iterator beginIt)
         for (auto it2 = beginIt; it2 != it; ++it2) {
             m_reservationToGroupMap.insert(*it2, groupIt.key());
         }
-        g.setName(guessName(g));
+        if (g.hasAutomaticName() || g.name().isEmpty()) {
+            g.setName(guessName(g.elements()));
+        }
+        recomputeTripGroupTimes(g);
         qDebug() << "updating trip group" << g.name();
-        g.store(basePath() + groupIt.key() + QLatin1StringView(".json"));
+        g.store(fileForGroup(groupIt.key()));
         Q_EMIT tripGroupChanged(groupIt.key());
     }
 
@@ -470,6 +607,16 @@ void TripGroupManager::checkConsistency()
     }
     tgIds.clear();
 
+    // look for missing or otherwise corrupt begin/end times
+    for (auto it = m_tripGroups.begin(); it != m_tripGroups.end(); ++it) {
+        if (it.value().beginDateTime().isValid() && it.value().endDateTime().isValid() && it.value().beginDateTime() <= it.value().endDateTime()) {
+            continue;
+        }
+        qCDebug(Log) << "Fixing begin/end times for group" << it.value().name();
+        recomputeTripGroupTimes(it.value());
+        it.value().store(fileForGroup(it.key()));
+    }
+
     // look for nested groups
     std::copy(m_tripGroups.keyBegin(), m_tripGroups.keyEnd(), std::back_inserter(tgIds));
     std::sort(tgIds.begin(), tgIds.end(), [this](const auto &lhs, const auto &rhs) {
@@ -499,11 +646,11 @@ static QString destinationName(const QVariant &loc)
     return LocationUtil::name(loc);
 }
 
-QString TripGroupManager::guessDestinationFromLodging(const TripGroup &g) const
+QString TripGroupManager::guessDestinationFromLodging(const QStringList &elements) const
 {
     // we assume that lodging indicates the actual destination, not a stopover location
     QStringList dests;
-    for (const auto &resId : g.elements()) {
+    for (const auto &resId : elements) {
         const auto res = m_resMgr->reservation(resId);
         if (!JsonLd::isA<LodgingReservation>(res)) {
             continue;
@@ -522,22 +669,22 @@ QString TripGroupManager::guessDestinationFromLodging(const TripGroup &g) const
         // TODO consider the country if that differs from where we started from
     }
 
-    return dests.join(QLatin1StringView(" - "));
+    return dests.join(" - "_L1);
 }
 
-bool TripGroupManager::isRoundTrip(const TripGroup& g) const
+bool TripGroupManager::isRoundTrip(const QStringList &elements) const
 {
-    const auto depId = g.elements().at(0);
-    const auto arrId = g.elements().constLast();
+    const auto &depId = elements.at(0);
+    const auto &arrId = elements.constLast();
     const auto dep = LocationUtil::departureLocation(m_resMgr->reservation(depId));
     const auto arr = LocationUtil::arrivalLocation(m_resMgr->reservation(arrId));
     return LocationUtil::isSameLocation(dep, arr, LocationUtil::CityLevel);
 }
 
-QString TripGroupManager::guessDestinationFromTransportTimeGap(const TripGroup &g) const
+QString TripGroupManager::guessDestinationFromTransportTimeGap(const QStringList &elements) const
 {
     // we must only do this for return trips
-    if (!isRoundTrip(g)) {
+    if (!isRoundTrip(elements)) {
         return {};
     }
 
@@ -546,7 +693,7 @@ QString TripGroupManager::guessDestinationFromTransportTimeGap(const TripGroup &
     QString destName;
     qint64 maxLength = 0;
 
-    for (const auto &resId : g.elements()) {
+    for (const auto &resId : elements) {
         const auto res = m_resMgr->reservation(resId);
         if (!LocationUtil::isLocationChange(res)) {
             continue;
@@ -569,20 +716,24 @@ QString TripGroupManager::guessDestinationFromTransportTimeGap(const TripGroup &
     return destName;
 }
 
-QString TripGroupManager::guessName(const TripGroup& g) const
+QString TripGroupManager::guessName(const QStringList &elements) const
 {
+    if (elements.isEmpty()) {
+        return {};
+    }
+
     // part 1: the destination of the trip
-    QString dest = guessDestinationFromLodging(g);
+    QString dest = guessDestinationFromLodging(elements);
     if (dest.isEmpty()) {
-        dest = guessDestinationFromTransportTimeGap(g);
+        dest = guessDestinationFromTransportTimeGap(elements);
     }
     if (dest.isEmpty()) {
         // two fallback cases: round-trips and one-way trips
-        const auto beginLoc = LocationUtil::departureLocation(m_resMgr->reservation(g.elements().at(0)));
-        const auto endLoc = LocationUtil::arrivalLocation(m_resMgr->reservation(g.elements().constLast()));
+        const auto beginLoc = LocationUtil::departureLocation(m_resMgr->reservation(elements.at(0)));
+        const auto endLoc = LocationUtil::arrivalLocation(m_resMgr->reservation(elements.constLast()));
         if (LocationUtil::isSameLocation(beginLoc, endLoc, LocationUtil::CityLevel)) {
-            const auto middleIdx = (g.elements().size() - 1 + (g.elements().size() % 2)) / 2;
-            const auto middleRes = m_resMgr->reservation(g.elements().at(middleIdx));
+            const auto middleIdx = (elements.size() - 1 + (elements.size() % 2)) / 2;
+            const auto middleRes = m_resMgr->reservation(elements.at(middleIdx));
             if (LocationUtil::isLocationChange(middleRes)) {
                 dest = destinationName(LocationUtil::arrivalLocation(middleRes));
             } else {
@@ -593,19 +744,179 @@ QString TripGroupManager::guessName(const TripGroup& g) const
             dest = destinationName(endLoc);
         }
     }
+    // if none of the above worked, take anything we can find
+    for (const auto &resId : elements) {
+        if (!dest.isEmpty()) {
+            break;
+        }
+        const auto res = m_resMgr->reservation(resId);
+        dest = LocationUtil::name(LocationUtil::location(res));
+        if (dest.isEmpty()) {
+            dest = LocationUtil::name(LocationUtil::arrivalLocation(res));
+        }
+        if (dest.isEmpty() && JsonLd::isA<EventReservation>(res)) {
+            dest = res.value<EventReservation>().reservationFor().value<Event>().name();
+        }
+        if (dest.isEmpty() && JsonLd::isA<LodgingReservation>(res)) {
+            dest = res.value<EventReservation>().reservationFor().value<LodgingBusiness>().name();
+        }
+        if (dest.isEmpty()) {
+            dest = LocationUtil::name(LocationUtil::departureLocation(res));
+        }
+    }
+
 
     // part 2: the time range of the trip
     // three cases: within 1 month, crossing a month boundary in one year, crossing a year boundary
-    const auto beginDt = SortUtil::startDateTime(m_resMgr->reservation(g.elements().at(0)));
-    const auto endDt = SortUtil::endDateTime(m_resMgr->reservation(g.elements().constLast()));
-    Q_ASSERT(beginDt.daysTo(endDt) <= MaximumTripDuration);
-    if (beginDt.date().year() == endDt.date().year()) {
-        if (beginDt.date().month() == endDt.date().month()) {
-            return i18nc("%1 is destination, %2 is the standalone month name, %3 is the year", "%1 (%2 %3)", dest, QLocale().standaloneMonthName(beginDt.date().month(), QLocale::LongFormat), beginDt.date().toString(QStringLiteral("yyyy")));
+    const auto beginDt = SortUtil::startDateTime(m_resMgr->reservation(elements.at(0)));
+    const auto endDt = SortUtil::endDateTime(m_resMgr->reservation(elements.constLast()));
+    if (beginDt.date().year() == endDt.date().year() || !endDt.isValid()) {
+        if (beginDt.date().month() == endDt.date().month() || !endDt.isValid()) {
+            return i18nc("%1 is destination, %2 is the standalone month name, %3 is the year", "%1 (%2 %3)", dest, QLocale().standaloneMonthName(beginDt.date().month(), QLocale::LongFormat), beginDt.date().toString(u"yyyy"));
         }
-        return i18nc("%1 is destination, %2 and %3 are the standalone month names and %4 is the year", "%1 (%2/%3 %4)", dest, QLocale().monthName(beginDt.date().month(), QLocale::LongFormat), QLocale().standaloneMonthName(endDt.date().month(), QLocale::LongFormat), beginDt.date().toString(QStringLiteral("yyyy")));
+        return i18nc("%1 is destination, %2 and %3 are the standalone month names and %4 is the year", "%1 (%2/%3 %4)", dest, QLocale().monthName(beginDt.date().month(), QLocale::LongFormat), QLocale().standaloneMonthName(endDt.date().month(), QLocale::LongFormat), beginDt.date().toString(u"yyyy"));
     }
-    return i18nc("%1 is destination, %2 and %3 are years", "%1 (%2/%3)", dest, beginDt.date().toString(QStringLiteral("yyyy")), endDt.date().toString(QStringLiteral("yyyy")));
+    return i18nc("%1 is destination, %2 and %3 are years", "%1 (%2/%3)", dest, beginDt.date().toString(u"yyyy"), endDt.date().toString(u"yyyy"));
+}
+
+bool TripGroupManager::recomputeTripGroupTimes(TripGroup &tg) const
+{
+    if (tg.elements().isEmpty() && (tg.beginDateTime().isValid() || tg.endDateTime().isValid())) {
+        tg.setBeginDateTime({});
+        tg.setEndDateTime({});
+        return true;
+    }
+    if (tg.elements().isEmpty()) {
+        return false;
+    }
+
+    auto res = m_resMgr->reservation(tg.elements().constFirst());
+    auto dt = KItinerary::SortUtil::startDateTime(res);
+
+    auto transfer = m_transferMgr->transfer(tg.elements().constFirst(), Transfer::Before);
+    if (transfer.state() == Transfer::Selected && transfer.journey().scheduledDepartureTime().isValid()) {
+        dt = std::min(dt, transfer.journey().scheduledDepartureTime());
+    } else if (transfer.state() == Transfer::Pending && transfer.anchorTime().isValid()) {
+        dt = std::min(dt, transfer.anchorTime().addSecs(-transfer.anchorTimeDelta()));
+    }
+    auto change = dt != tg.beginDateTime();
+    tg.setBeginDateTime(dt);
+
+    res = m_resMgr->reservation(tg.elements().constLast());
+    dt = KItinerary::SortUtil::endDateTime(res);
+
+    transfer = m_transferMgr->transfer(tg.elements().constLast(), Transfer::After);
+    if (transfer.state() == Transfer::Selected && transfer.journey().scheduledArrivalTime().isValid()) {
+        dt = std::max(dt, transfer.journey().scheduledArrivalTime());
+    } else if (transfer.state() == Transfer::Pending && transfer.anchorTime().isValid()) {
+        dt = std::min(dt, transfer.anchorTime().addSecs(transfer.anchorTimeDelta()));
+    }
+    change |= dt != tg.endDateTime();
+    tg.setEndDateTime(dt);
+    return change;
+}
+
+QString TripGroupManager::merge(const QString &tgId1, const QString &tgId2, const QString &newName)
+{
+    qCDebug(Log) << tgId1 << tgId2 << newName;
+
+    const auto tg1 = tripGroup(tgId1);
+    const auto tg2 = tripGroup(tgId2);
+
+    QStringList elements;
+    elements.reserve(tg1.elements().size() + tg2.elements().size());
+    elements.append(tg1.elements());
+    elements.append(tg2.elements());
+    std::sort(elements.begin(), elements.end(), [this](const auto &lhs, const auto &rhs) {
+        return SortUtil::isBefore(m_resMgr->reservation(lhs), m_resMgr->reservation(rhs));
+    });
+
+    for (const auto &id : elements) {
+        m_reservationToGroupMap[id] = tgId1;
+    }
+
+    TripGroup group;
+    group.setElements(elements);
+    group.setIsAutomaticallyGrouped(false);
+
+    group.setName(guessName(elements));
+    if (group.name() != newName && !newName.isEmpty()) {
+        group.setName(newName);
+        group.setNameIsAutomatic(false);
+    }
+
+    group.setMatrixRoomId(tg1.matrixRoomId().isEmpty() ? tg2.matrixRoomId() : tg1.matrixRoomId());
+
+    recomputeTripGroupTimes(group);
+    m_tripGroups[tgId1] = group;
+    group.store(fileForGroup(tgId1));
+
+    m_tripGroups.remove(tgId2);
+    if (!QFile::remove(fileForGroup(tgId2))) {
+        qCWarning(Log) << "Failed to delete trip group file!" << tgId2;
+    }
+
+    Q_EMIT tripGroupRemoved(tgId2);
+    Q_EMIT tripGroupChanged(tgId1);
+
+    return tgId1;
+}
+
+QString TripGroupManager::createGroup(const QStringList &elements, const QString &name)
+{
+    qCDebug(Log) << elements << name;
+
+    // find groups that contain any of the selected elements
+    QStringList groupsToUpdate;
+    for (const auto &resId : elements) {
+        const auto it = m_reservationToGroupMap.constFind(resId);
+        if (it == m_reservationToGroupMap.cend()) {
+            continue;
+        }
+        if (!groupsToUpdate.contains(it.value())) {
+            groupsToUpdate.push_back(it.value());
+        }
+    }
+
+    // remove elements from other groups and mark those explicitly grouped
+    for (const auto &tgId :groupsToUpdate) {
+        auto &tg = m_tripGroups[tgId];
+        auto elems = tg.elements();
+        for (const auto &resId : elements) {
+            elems.removeAll(resId);
+        }
+        // TODO can elems becomes empty here?
+        tg.setElements(elems);
+        tg.setIsAutomaticallyGrouped(false);
+        if (tg.hasAutomaticName()) {
+            tg.setName(guessName(tg.elements()));
+        }
+        recomputeTripGroupTimes(tg);
+        tg.store(fileForGroup(tgId));
+        Q_EMIT tripGroupChanged(tgId);
+    }
+
+    // create new group
+    TripGroup tg;
+    tg.setElements(elements);
+    tg.setIsAutomaticallyGrouped(false);
+    tg.setName(guessName(elements));
+    if (tg.name() != name && !name.isEmpty()) {
+        tg.setName(name);
+        tg.setNameIsAutomatic(false);
+    }
+    recomputeTripGroupTimes(tg);
+
+    const auto tgId = QUuid::createUuid().toString(QUuid::WithoutBraces);
+    m_tripGroups.insert(tgId, tg);
+    for (const auto &resId : elements) {
+        m_reservationToGroupMap.insert(resId, tgId);
+    }
+
+    tg.store(fileForGroup(tgId));
+    Q_EMIT tripGroupAdded(tgId);
+
+    return tgId;
 }
 
 #include "moc_tripgroupmanager.cpp"
