@@ -10,13 +10,7 @@
 #include "jniprimitivetypes.h"
 #include "jnitypes.h"
 
-#include <qglobal.h>
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <QAndroidJniObject>
-#else
 #include <QJniObject>
-using QAndroidJniObject = QJniObject;
-#endif
 
 namespace KAndroidExtras {
 
@@ -52,18 +46,18 @@ template <typename T> struct is_generic_wrapper : std::conditional_t<
  * Declare a JNI type to be convertible to a native type.
  * @param JniType A type declared with @p JNI_TYPE.
  * @param NativeType A C++ type @p JniType can be converted to/from.
- * @param FromJniFn Code converting a @c QAndroidJniObject @c value to @p NativeType.
- * @param ToJniCode converting a @p NativeType @c value to a QAndroidJniObject.
+ * @param FromJniFn Code converting a @c QJniObject @c value to @p NativeType.
+ * @param ToJniCode converting a @p NativeType @c value to a QJniObject.
  */
 #define JNI_DECLARE_CONVERTER(JniType, NativeType, FromJniFn, ToJniFn) \
 namespace Jni { \
 template <> struct converter<NativeType> { \
     typedef JniType type; \
-    static inline QAndroidJniObject convert(const NativeType &value) { return (ToJniFn); } \
+    static inline QJniObject convert(const NativeType &value) { return (ToJniFn); } \
 }; \
 template <> struct converter<JniType> { \
     typedef NativeType type; \
-    static inline NativeType convert(const QAndroidJniObject &value) { return (FromJniFn); } \
+    static inline NativeType convert(const QJniObject &value) { return (FromJniFn); } \
 }; \
 }
 
